==> wow-core-1.9.5-sources.jar: me/ahoo/wow/command/wait/NotifierFilters.kt <==
/*
 * Copyright [2021-present] [ahoo wang <ahoowang@qq.com> (https://github.com/Ahoo-Wang)].
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *      http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package me.ahoo.wow.command.wait

import me.ahoo.wow.api.annotation.ORDER_FIRST
import me.ahoo.wow.api.annotation.Order
import me.ahoo.wow.command.ServerCommandExchange
import me.ahoo.wow.event.DomainEventExchange
import me.ahoo.wow.event.EventStreamExchange
import me.ahoo.wow.eventsourcing.snapshot.SnapshotDispatcher
import me.ahoo.wow.messaging.handler.Filter
import me.ahoo.wow.messaging.handler.FilterChain
import me.ahoo.wow.messaging.handler.FilterType
import me.ahoo.wow.messaging.handler.MessageExchange
import me.ahoo.wow.modeling.command.CommandDispatcher
import me.ahoo.wow.projection.ProjectionDispatcher
import reactor.core.publisher.Mono

abstract class AbstractNotifierFilter<T : MessageExchange<*, *>>(
    private val processingStage: CommandStage,
    private val commandWaitNotifier: CommandWaitNotifier,
) : Filter<T> {
    override fun filter(
        exchange: T,
        next: FilterChain<T>,
    ): Mono<Void> {
        return next.filter(exchange)
            .thenNotifyAndForget(commandWaitNotifier, processingStage, exchange)
    }
}

@FilterType(CommandDispatcher::class)
@Order(ORDER_FIRST)
class ProcessedNotifierFilter(
    commandWaitNotifier: CommandWaitNotifier,
) : AbstractNotifierFilter<ServerCommandExchange<Any>>(CommandStage.PROCESSED, commandWaitNotifier)

@FilterType(SnapshotDispatcher::class)
@Order(ORDER_FIRST)
class SnapshotNotifierFilter(
    commandWaitNotifier: CommandWaitNotifier,
) : AbstractNotifierFilter<EventStreamExchange>(CommandStage.SNAPSHOT, commandWaitNotifier)

@FilterType(ProjectionDispatcher::class)
@Order(ORDER_FIRST)
class ProjectedNotifierFilter(
    commandWaitNotifier: CommandWaitNotifier,
) : AbstractNotifierFilter<DomainEventExchange<Any>>(CommandStage.PROJECTED, commandWaitNotifier)

==> wow-core-1.10.0-sources.jar: me/ahoo/wow/command/wait/NotifierFilters.kt <==
/*
 * Copyright [2021-present] [ahoo wang <ahoowang@qq.com> (https://github.com/Ahoo-Wang)].
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *      http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package me.ahoo.wow.command.wait

import me.ahoo.wow.api.annotation.ORDER_FIRST
import me.ahoo.wow.api.annotation.Order
import me.ahoo.wow.command.ServerCommandExchange
import me.ahoo.wow.event.DomainEventExchange
import me.ahoo.wow.event.EventStreamExchange
import me.ahoo.wow.eventsourcing.snapshot.SnapshotDispatcher
import me.ahoo.wow.messaging.handler.Filter
import me.ahoo.wow.messaging.handler.FilterChain
import me.ahoo.wow.messaging.handler.FilterType
import me.ahoo.wow.messaging.handler.MessageExchange
import me.ahoo.wow.modeling.command.CommandDispatcher
import me.ahoo.wow.projection.ProjectionDispatcher
import reactor.core.publisher.Mono

abstract class AbstractNotifierFilter<T : MessageExchange<*, *>>(
    private val processingStage: CommandStage,
    private val commandWaitNotifier: CommandWaitNotifier
) : Filter<T> {
    override fun filter(
        exchange: T,
        next: FilterChain<T>
    ): Mono<Void> {
        return next.filter(exchange)
            .thenNotifyAndForget(commandWaitNotifier, processingStage, exchange)
    }
}

@FilterType(CommandDispatcher::class)
@Order(ORDER_FIRST)
class ProcessedNotifierFilter(
    commandWaitNotifier: CommandWaitNotifier
) : AbstractNotifierFilter<ServerCommandExchange<*>>(CommandStage.PROCESSED, commandWaitNotifier)

@FilterType(SnapshotDispatcher::class)
@Order(ORDER_FIRST)
class SnapshotNotifierFilter(
    commandWaitNotifier: CommandWaitNotifier
) : AbstractNotifierFilter<EventStreamExchange>(CommandStage.SNAPSHOT, commandWaitNotifier)

@FilterType(ProjectionDispatcher::class)
@Order(ORDER_FIRST)
class ProjectedNotifierFilter(
    commandWaitNotifier: CommandWaitNotifier
) : AbstractNotifierFilter<DomainEventExchange<Any>>(CommandStage.PROJECTED, commandWaitNotifier)
